--[[
	INFORMATION

	Creator: Relozu
]]

----------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------- LOCAL ATTRIBUTES AND FUNCTIONS ----------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------

local relozu_terrain_material_water = { }

local gModId
local gExtraSettingsFile   = "extra_mod_settings.lua: "
local gErrMsgIgnoreSetting = " - Setting ignored."
local gErrMsgIgnoreTable   = " - Table ignored."
local gDefaultParam        = 1
local gOrderBaseValue      = 6000

local gDataTableTerrainMaterial = { }
local gDataTableGroundTex       = { }
local gModParams                = { }
local gExtraSettings            = { }

local gHiddenSettings = {
	allowFilteringWaterDarkGrayStale = false,
}

-- Water colors
local gWaterColor = {
	temperateDry = { id = 0, name = "temperate_dry", },
	tropical     = { id = 1, name = "tropical", },
	darkGray     = { id = 2, name = "dark_gray", },
	blue         = { id = 3, name = "blue", },
	greenBlue    = { id = 4, name = "green_blue", },
	green        = { id = 5, name = "green", },
	lightBrown   = { id = 6, name = "light_brown", },
	brownRed     = { id = 7, name = "brown_red", },
	foam         = { id = 8, name = "foam", },
}

local gParamsFoamHOffset = {
	"0",
	"0.1",
	"0.2",
	"0.3",
	"0.4",
	"0.5",
}

-- Water types
local gWaterType = {
	fresh  = { id = 0, name = "fresh", },
	stale  = { id = 1, name = "stale", },
	small  = { id = 2, name = "small_low", },
	low    = { id = 3, name = "low", },
	medium = { id = 4, name = "medium", },
	high   = { id = 5, name = "high", },
	
	waveHeightLow    = { name = "wave_height_low" },
	waveHeightMedium = { name = "wave_height_medium" },
	waveHeightHigh   = { name = "wave_height_high" },
}

local gParamsWaveHeight = {
	_("pSmall"),
	_("pBig"),
}

local function setModId(modId)
	if ((type(modId) == "string") and (string.len(modId) > 0)) then
		gModId = modId
	end
end

local function getModPrefix()
	if (gModId ~= nil) then
		return "[" .. gModId .. "] "
	else
		return "[relozu_terrain_material_water] "
	end
end

local function printWarningMessage(errorMessage)
	print(getModPrefix() .. errorMessage)
end

local function prepareModParams(modParams)
	if (modParams == nil) then
		modParams = { }
	end

	if (modParams.kRelozu02TemperateDryFresh            == nil) then modParams.kRelozu02TemperateDryFresh            = 1             end
	if (modParams.kRelozu02TemperateDryStale            == nil) then modParams.kRelozu02TemperateDryStale            = 1             end
	if (modParams.kRelozu02TemperateDrySmall            == nil) then modParams.kRelozu02TemperateDrySmall            = gDefaultParam end
	if (modParams.kRelozu02TemperateDryLow              == nil) then modParams.kRelozu02TemperateDryLow              = 1             end
	if (modParams.kRelozu02TemperateDryMedium           == nil) then modParams.kRelozu02TemperateDryMedium           = 1             end
	if (modParams.kRelozu02TemperateDryHigh             == nil) then modParams.kRelozu02TemperateDryHigh             = 1             end
	if (modParams.kRelozu02WaveHeightTemperateDryLow    == nil) then modParams.kRelozu02WaveHeightTemperateDryLow    = 1             end
	if (modParams.kRelozu02WaveHeightTemperateDryMedium == nil) then modParams.kRelozu02WaveHeightTemperateDryMedium = 1             end
	if (modParams.kRelozu02WaveHeightTemperateDryHigh   == nil) then modParams.kRelozu02WaveHeightTemperateDryHigh   = 1             end
	if (modParams.kRelozu02TropicalFresh                == nil) then modParams.kRelozu02TropicalFresh                = 1             end
	if (modParams.kRelozu02TropicalStale                == nil) then modParams.kRelozu02TropicalStale                = 1             end
	if (modParams.kRelozu02TropicalSmall                == nil) then modParams.kRelozu02TropicalSmall                = gDefaultParam end
	if (modParams.kRelozu02TropicalLow                  == nil) then modParams.kRelozu02TropicalLow                  = 1             end
	if (modParams.kRelozu02TropicalMedium               == nil) then modParams.kRelozu02TropicalMedium               = 1             end
	if (modParams.kRelozu02TropicalHigh                 == nil) then modParams.kRelozu02TropicalHigh                 = 1             end
	if (modParams.kRelozu02WaveHeightTropicalLow        == nil) then modParams.kRelozu02WaveHeightTropicalLow        = 1             end
	if (modParams.kRelozu02WaveHeightTropicalMedium     == nil) then modParams.kRelozu02WaveHeightTropicalMedium     = 1             end
	if (modParams.kRelozu02WaveHeightTropicalHigh       == nil) then modParams.kRelozu02WaveHeightTropicalHigh       = 1             end
	if (modParams.kRelozu02DarkGrayFresh                == nil) then modParams.kRelozu02DarkGrayFresh                = gDefaultParam end
	if (modParams.kRelozu02DarkGrayStale                == nil) then modParams.kRelozu02DarkGrayStale                = gDefaultParam end
	if (modParams.kRelozu02DarkGraySmall                == nil) then modParams.kRelozu02DarkGraySmall                = gDefaultParam end
	if (modParams.kRelozu02DarkGrayLow                  == nil) then modParams.kRelozu02DarkGrayLow                  = gDefaultParam end
	if (modParams.kRelozu02DarkGrayMedium               == nil) then modParams.kRelozu02DarkGrayMedium               = gDefaultParam end
	if (modParams.kRelozu02DarkGrayHigh                 == nil) then modParams.kRelozu02DarkGrayHigh                 = gDefaultParam end
	if (modParams.kRelozu02WaveHeightDarkGrayLow        == nil) then modParams.kRelozu02WaveHeightDarkGrayLow        = 1             end
	if (modParams.kRelozu02WaveHeightDarkGrayMedium     == nil) then modParams.kRelozu02WaveHeightDarkGrayMedium     = 1             end
	if (modParams.kRelozu02WaveHeightDarkGrayHigh       == nil) then modParams.kRelozu02WaveHeightDarkGrayHigh       = 1             end
	if (modParams.kRelozu02BlueFresh                    == nil) then modParams.kRelozu02BlueFresh                    = 1             end
	if (modParams.kRelozu02BlueStale                    == nil) then modParams.kRelozu02BlueStale                    = 1             end
	if (modParams.kRelozu02BlueSmall                    == nil) then modParams.kRelozu02BlueSmall                    = gDefaultParam end
	if (modParams.kRelozu02BlueLow                      == nil) then modParams.kRelozu02BlueLow                      = 1             end
	if (modParams.kRelozu02BlueMedium                   == nil) then modParams.kRelozu02BlueMedium                   = 1             end
	if (modParams.kRelozu02BlueHigh                     == nil) then modParams.kRelozu02BlueHigh                     = 1             end
	if (modParams.kRelozu02WaveHeightBlueLow            == nil) then modParams.kRelozu02WaveHeightBlueLow            = 1             end
	if (modParams.kRelozu02WaveHeightBlueMedium         == nil) then modParams.kRelozu02WaveHeightBlueMedium         = 1             end
	if (modParams.kRelozu02WaveHeightBlueHigh           == nil) then modParams.kRelozu02WaveHeightBlueHigh           = 1             end
	if (modParams.kRelozu02GreenBlueFresh               == nil) then modParams.kRelozu02GreenBlueFresh               = 1             end
	if (modParams.kRelozu02GreenBlueStale               == nil) then modParams.kRelozu02GreenBlueStale               = 1             end
	if (modParams.kRelozu02GreenBlueSmall               == nil) then modParams.kRelozu02GreenBlueSmall               = gDefaultParam end
	if (modParams.kRelozu02GreenBlueLow                 == nil) then modParams.kRelozu02GreenBlueLow                 = 1             end
	if (modParams.kRelozu02GreenBlueMedium              == nil) then modParams.kRelozu02GreenBlueMedium              = 1             end
	if (modParams.kRelozu02GreenBlueHigh                == nil) then modParams.kRelozu02GreenBlueHigh                = 1             end
	if (modParams.kRelozu02WaveHeightGreenBlueLow       == nil) then modParams.kRelozu02WaveHeightGreenBlueLow       = 1             end
	if (modParams.kRelozu02WaveHeightGreenBlueMedium    == nil) then modParams.kRelozu02WaveHeightGreenBlueMedium    = 1             end
	if (modParams.kRelozu02WaveHeightGreenBlueHigh      == nil) then modParams.kRelozu02WaveHeightGreenBlueHigh      = 1             end
	if (modParams.kRelozu02GreenFresh                   == nil) then modParams.kRelozu02GreenFresh                   = 1             end
	if (modParams.kRelozu02GreenStale                   == nil) then modParams.kRelozu02GreenStale                   = 1             end
	if (modParams.kRelozu02GreenSmall                   == nil) then modParams.kRelozu02GreenSmall                   = gDefaultParam end
	if (modParams.kRelozu02GreenLow                     == nil) then modParams.kRelozu02GreenLow                     = 1             end
	if (modParams.kRelozu02GreenMedium                  == nil) then modParams.kRelozu02GreenMedium                  = 1             end
	if (modParams.kRelozu02GreenHigh                    == nil) then modParams.kRelozu02GreenHigh                    = 1             end
	if (modParams.kRelozu02WaveHeightGreenLow           == nil) then modParams.kRelozu02WaveHeightGreenLow           = 1             end
	if (modParams.kRelozu02WaveHeightGreenMedium        == nil) then modParams.kRelozu02WaveHeightGreenMedium        = 1             end
	if (modParams.kRelozu02WaveHeightGreenHigh          == nil) then modParams.kRelozu02WaveHeightGreenHigh          = 1             end
	if (modParams.kRelozu02LightBrownFresh              == nil) then modParams.kRelozu02LightBrownFresh              = 1             end
	if (modParams.kRelozu02LightBrownStale              == nil) then modParams.kRelozu02LightBrownStale              = 1             end
	if (modParams.kRelozu02LightBrownSmall              == nil) then modParams.kRelozu02LightBrownSmall              = gDefaultParam end
	if (modParams.kRelozu02LightBrownLow                == nil) then modParams.kRelozu02LightBrownLow                = 1             end
	if (modParams.kRelozu02LightBrownMedium             == nil) then modParams.kRelozu02LightBrownMedium             = 1             end
	if (modParams.kRelozu02LightBrownHigh               == nil) then modParams.kRelozu02LightBrownHigh               = 1             end
	if (modParams.kRelozu02WaveHeightLightBrownLow      == nil) then modParams.kRelozu02WaveHeightLightBrownLow      = 1             end
	if (modParams.kRelozu02WaveHeightLightBrownMedium   == nil) then modParams.kRelozu02WaveHeightLightBrownMedium   = 1             end
	if (modParams.kRelozu02WaveHeightLightBrownHigh     == nil) then modParams.kRelozu02WaveHeightLightBrownHigh     = 1             end
	if (modParams.kRelozu02BrownRedFresh                == nil) then modParams.kRelozu02BrownRedFresh                = 1             end
	if (modParams.kRelozu02BrownRedStale                == nil) then modParams.kRelozu02BrownRedStale                = 1             end
	if (modParams.kRelozu02BrownRedSmall                == nil) then modParams.kRelozu02BrownRedSmall                = gDefaultParam end
	if (modParams.kRelozu02BrownRedLow                  == nil) then modParams.kRelozu02BrownRedLow                  = 1             end
	if (modParams.kRelozu02BrownRedMedium               == nil) then modParams.kRelozu02BrownRedMedium               = 1             end
	if (modParams.kRelozu02BrownRedHigh                 == nil) then modParams.kRelozu02BrownRedHigh                 = 1             end
	if (modParams.kRelozu02WaveHeightBrownRedLow        == nil) then modParams.kRelozu02WaveHeightBrownRedLow        = 1             end
	if (modParams.kRelozu02WaveHeightBrownRedMedium     == nil) then modParams.kRelozu02WaveHeightBrownRedMedium     = 1             end
	if (modParams.kRelozu02WaveHeightBrownRedHigh       == nil) then modParams.kRelozu02WaveHeightBrownRedHigh       = 1             end
	if (modParams.kRelozu02Foam01                       == nil) then modParams.kRelozu02Foam01                       = 1             end
	if (modParams.kRelozu02Foam01HOffset                == nil) then modParams.kRelozu02Foam01HOffset                = 0             end
	if (modParams.kRelozu02Foam02                       == nil) then modParams.kRelozu02Foam02                       = 1             end
	if (modParams.kRelozu02Foam02HOffset                == nil) then modParams.kRelozu02Foam02HOffset                = 0             end

	gModParams = table.copy(modParams)
end

local function adoptExtraSettings()
	local indexCurrent = -1
	local currentSettingsName = "unknown"
	local currentSettingsKey  = "unknown key"

	-- min, max, factor, return 1 as fallback, message
	local settingType = {
		waveHeight       = { 0,  1,   1,  false, gErrMsgIgnoreSetting },
		foamHOffset      = { 0,  1,   10, false, gErrMsgIgnoreSetting },
	}

	local function printDefaultErrorMessageExtra(source, errorMessage)
		printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. indexCurrent .. "]." .. source .. " (Index: " .. indexCurrent .. ", Key: \"" .. currentSettingsKey .. "\", Name: \"" .. currentSettingsName .. "\")" .. errorMessage)
	end

	local function checkDefaultSetting(settingTypeValue, newParam, oldParam, errorMessage)
		if (newParam ~= nil) then
			if (type(newParam) == "number") then
				if ((newParam >= settingTypeValue[1]) and (newParam <= settingTypeValue[2])) then
					return newParam * settingTypeValue[3]
				else
					printDefaultErrorMessageExtra(errorMessage, " - Accepted number range: n >= " .. settingTypeValue[1] .. "; n <= " .. settingTypeValue[2] .. " - Get: " .. newParam .. settingTypeValue[5])
				end
			else
				printDefaultErrorMessageExtra(errorMessage, " - Expected type: number - Get: " .. type(newParam) .. settingTypeValue[5])
			end
		end

		if ((newParam == nil) or (settingTypeValue[4] == false)) then
			return oldParam
		else
			return 1
		end
	end

	local function checkMaterial(newParam, oldParam, errorMessage)
		if (newParam ~= nil) then
			if(type(newParam) == "boolean") then
				if(newParam) then
					return 1
				else
					return 0
				end
			else
				printDefaultErrorMessageExtra(errorMessage, " - Expected type: boolean - Get: " .. type(newParam) .. " - true will be used as fallback")
			end
		elseif (newParam == nil) then
			return oldParam
		end
	
		return 1
	end

	local function checkSettings(extraSettings, index)
		indexCurrent        = index
		currentSettingsKey  = extraSettings.key
		currentSettingsName = extraSettings.param.name
	
		-- Terrain materials
		if(extraSettings.temperate_dry ~= nil) then
			gModParams.kRelozu02TemperateDryFresh  = checkMaterial(extraSettings.temperate_dry.fresh,  gModParams.kRelozu02TemperateDryFresh,  gWaterColor.temperateDry.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02TemperateDryStale  = checkMaterial(extraSettings.temperate_dry.stale,  gModParams.kRelozu02TemperateDryStale,  gWaterColor.temperateDry.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02TemperateDrySmall  = checkMaterial(extraSettings.temperate_dry.small,  gModParams.kRelozu02TemperateDrySmall,  gWaterColor.temperateDry.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02TemperateDryLow    = checkMaterial(extraSettings.temperate_dry.low,    gModParams.kRelozu02TemperateDryLow,    gWaterColor.temperateDry.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02TemperateDryMedium = checkMaterial(extraSettings.temperate_dry.medium, gModParams.kRelozu02TemperateDryMedium, gWaterColor.temperateDry.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02TemperateDryHigh   = checkMaterial(extraSettings.temperate_dry.high,   gModParams.kRelozu02TemperateDryHigh,   gWaterColor.temperateDry.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightTemperateDryLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.temperate_dry.wave_height_low,    gModParams.kRelozu02WaveHeightTemperateDryLow,    gWaterColor.temperateDry.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightTemperateDryMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.temperate_dry.wave_height_medium, gModParams.kRelozu02WaveHeightTemperateDryMedium, gWaterColor.temperateDry.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightTemperateDryHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.temperate_dry.wave_height_high,   gModParams.kRelozu02WaveHeightTemperateDryHigh,   gWaterColor.temperateDry.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.tropical ~= nil) then
			gModParams.kRelozu02TropicalFresh  = checkMaterial(extraSettings.tropical.fresh,  gModParams.kRelozu02TropicalFresh,  gWaterColor.tropical.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02TropicalStale  = checkMaterial(extraSettings.tropical.stale,  gModParams.kRelozu02TropicalStale,  gWaterColor.tropical.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02TropicalSmall  = checkMaterial(extraSettings.tropical.small,  gModParams.kRelozu02TropicalSmall,  gWaterColor.tropical.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02TropicalLow    = checkMaterial(extraSettings.tropical.low,    gModParams.kRelozu02TropicalLow,    gWaterColor.tropical.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02TropicalMedium = checkMaterial(extraSettings.tropical.medium, gModParams.kRelozu02TropicalMedium, gWaterColor.tropical.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02TropicalHigh   = checkMaterial(extraSettings.tropical.high,   gModParams.kRelozu02TropicalHigh,   gWaterColor.tropical.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightTropicalLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.tropical.wave_height_low,    gModParams.kRelozu02WaveHeightTropicalLow,    gWaterColor.tropical.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightTropicalMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.tropical.wave_height_medium, gModParams.kRelozu02WaveHeightTropicalMedium, gWaterColor.tropical.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightTropicalHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.tropical.wave_height_high,   gModParams.kRelozu02WaveHeightTropicalHigh,   gWaterColor.tropical.name .. "." .. gWaterType.waveHeightHigh.name)
		end

		if(extraSettings.dark_gray ~= nil) then
			gModParams.kRelozu02DarkGrayFresh  = checkMaterial(extraSettings.dark_gray.fresh,  gModParams.kRelozu02DarkGrayFresh,  gWaterColor.darkGray.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02DarkGrayStale  = checkMaterial(extraSettings.dark_gray.stale,  gModParams.kRelozu02DarkGrayStale,  gWaterColor.darkGray.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02DarkGraySmall  = checkMaterial(extraSettings.dark_gray.small,  gModParams.kRelozu02DarkGraySmall,  gWaterColor.darkGray.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02DarkGrayLow    = checkMaterial(extraSettings.dark_gray.low,    gModParams.kRelozu02DarkGrayLow,    gWaterColor.darkGray.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02DarkGrayMedium = checkMaterial(extraSettings.dark_gray.medium, gModParams.kRelozu02DarkGrayMedium, gWaterColor.darkGray.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02DarkGrayHigh   = checkMaterial(extraSettings.dark_gray.high,   gModParams.kRelozu02DarkGrayHigh,   gWaterColor.darkGray.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightDarkGrayLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.dark_gray.wave_height_low,    gModParams.kRelozu02WaveHeightDarkGrayLow,    gWaterColor.darkGray.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightDarkGrayMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.dark_gray.wave_height_medium, gModParams.kRelozu02WaveHeightDarkGrayMedium, gWaterColor.darkGray.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightDarkGrayHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.dark_gray.wave_height_high,   gModParams.kRelozu02WaveHeightDarkGrayHigh,   gWaterColor.darkGray.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.blue ~= nil) then
			gModParams.kRelozu02BlueFresh  = checkMaterial(extraSettings.blue.fresh,  gModParams.kRelozu02BlueFresh,  gWaterColor.blue.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02BlueStale  = checkMaterial(extraSettings.blue.stale,  gModParams.kRelozu02BlueStale,  gWaterColor.blue.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02BlueSmall  = checkMaterial(extraSettings.blue.small,  gModParams.kRelozu02BlueSmall,  gWaterColor.blue.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02BlueLow    = checkMaterial(extraSettings.blue.low,    gModParams.kRelozu02BlueLow,    gWaterColor.blue.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02BlueMedium = checkMaterial(extraSettings.blue.medium, gModParams.kRelozu02BlueMedium, gWaterColor.blue.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02BlueHigh   = checkMaterial(extraSettings.blue.high,   gModParams.kRelozu02BlueHigh,   gWaterColor.blue.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightBlueLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.blue.wave_height_low,    gModParams.kRelozu02WaveHeightBlueLow,    gWaterColor.blue.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightBlueMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.blue.wave_height_medium, gModParams.kRelozu02WaveHeightBlueMedium, gWaterColor.blue.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightBlueHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.blue.wave_height_high,   gModParams.kRelozu02WaveHeightBlueHigh,   gWaterColor.blue.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.green_blue ~= nil) then
			gModParams.kRelozu02GreenBlueFresh  = checkMaterial(extraSettings.green_blue.fresh,  gModParams.kRelozu02GreenBlueFresh,  gWaterColor.greenBlue.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02GreenBlueStale  = checkMaterial(extraSettings.green_blue.stale,  gModParams.kRelozu02GreenBlueStale,  gWaterColor.greenBlue.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02GreenBlueSmall  = checkMaterial(extraSettings.green_blue.small,  gModParams.kRelozu02GreenBlueSmall,  gWaterColor.greenBlue.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02GreenBlueLow    = checkMaterial(extraSettings.green_blue.low,    gModParams.kRelozu02GreenBlueLow,    gWaterColor.greenBlue.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02GreenBlueMedium = checkMaterial(extraSettings.green_blue.medium, gModParams.kRelozu02GreenBlueMedium, gWaterColor.greenBlue.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02GreenBlueHigh   = checkMaterial(extraSettings.green_blue.high,   gModParams.kRelozu02GreenBlueHigh,   gWaterColor.greenBlue.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightGreenBlueLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.green_blue.wave_height_low,    gModParams.kRelozu02WaveHeightGreenBlueLow,    gWaterColor.greenBlue.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightGreenBlueMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.green_blue.wave_height_medium, gModParams.kRelozu02WaveHeightGreenBlueMedium, gWaterColor.greenBlue.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightGreenBlueHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.green_blue.wave_height_high,   gModParams.kRelozu02WaveHeightGreenBlueHigh,   gWaterColor.greenBlue.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.green ~= nil) then
			gModParams.kRelozu02GreenFresh  = checkMaterial(extraSettings.green.fresh,  gModParams.kRelozu02GreenFresh,  gWaterColor.green.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02GreenStale  = checkMaterial(extraSettings.green.stale,  gModParams.kRelozu02GreenStale,  gWaterColor.green.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02GreenSmall  = checkMaterial(extraSettings.green.small,  gModParams.kRelozu02GreenSmall,  gWaterColor.green.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02GreenLow    = checkMaterial(extraSettings.green.low,    gModParams.kRelozu02GreenLow,    gWaterColor.green.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02GreenMedium = checkMaterial(extraSettings.green.medium, gModParams.kRelozu02GreenMedium, gWaterColor.green.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02GreenHigh   = checkMaterial(extraSettings.green.high,   gModParams.kRelozu02GreenHigh,   gWaterColor.green.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightGreenLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.green.wave_height_low,    gModParams.kRelozu02WaveHeightGreenLow,    gWaterColor.green.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightGreenMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.green.wave_height_medium, gModParams.kRelozu02WaveHeightGreenMedium, gWaterColor.green.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightGreenHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.green.wave_height_high,   gModParams.kRelozu02WaveHeightGreenHigh,   gWaterColor.green.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.light_brown ~= nil) then
			gModParams.kRelozu02LightBrownFresh  = checkMaterial(extraSettings.light_brown.fresh,  gModParams.kRelozu02LightBrownFresh,  gWaterColor.lightBrown.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02LightBrownStale  = checkMaterial(extraSettings.light_brown.stale,  gModParams.kRelozu02LightBrownStale,  gWaterColor.lightBrown.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02LightBrownSmall  = checkMaterial(extraSettings.light_brown.small,  gModParams.kRelozu02LightBrownSmall,  gWaterColor.lightBrown.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02LightBrownLow    = checkMaterial(extraSettings.light_brown.low,    gModParams.kRelozu02LightBrownLow,    gWaterColor.lightBrown.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02LightBrownMedium = checkMaterial(extraSettings.light_brown.medium, gModParams.kRelozu02LightBrownMedium, gWaterColor.lightBrown.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02LightBrownHigh   = checkMaterial(extraSettings.light_brown.high,   gModParams.kRelozu02LightBrownHigh,   gWaterColor.lightBrown.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightLightBrownLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.light_brown.wave_height_low,    gModParams.kRelozu02WaveHeightLightBrownLow,    gWaterColor.lightBrown.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightLightBrownMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.light_brown.wave_height_medium, gModParams.kRelozu02WaveHeightLightBrownMedium, gWaterColor.lightBrown.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightLightBrownHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.light_brown.wave_height_high,   gModParams.kRelozu02WaveHeightLightBrownHigh,   gWaterColor.lightBrown.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.brown_red ~= nil) then
			gModParams.kRelozu02BrownRedFresh  = checkMaterial(extraSettings.brown_red.fresh,  gModParams.kRelozu02BrownRedFresh,  gWaterColor.brownRed.name .. "." .. gWaterType.fresh.name)
			gModParams.kRelozu02BrownRedStale  = checkMaterial(extraSettings.brown_red.stale,  gModParams.kRelozu02BrownRedStale,  gWaterColor.brownRed.name .. "." .. gWaterType.stale.name)
			gModParams.kRelozu02BrownRedSmall  = checkMaterial(extraSettings.brown_red.small,  gModParams.kRelozu02BrownRedSmall,  gWaterColor.brownRed.name .. "." .. gWaterType.small.name)
			gModParams.kRelozu02BrownRedLow    = checkMaterial(extraSettings.brown_red.low,    gModParams.kRelozu02BrownRedLow,    gWaterColor.brownRed.name .. "." .. gWaterType.low.name)
			gModParams.kRelozu02BrownRedMedium = checkMaterial(extraSettings.brown_red.medium, gModParams.kRelozu02BrownRedMedium, gWaterColor.brownRed.name .. "." .. gWaterType.medium.name)
			gModParams.kRelozu02BrownRedHigh   = checkMaterial(extraSettings.brown_red.high,   gModParams.kRelozu02BrownRedHigh,   gWaterColor.brownRed.name .. "." .. gWaterType.high.name)
	
			gModParams.kRelozu02WaveHeightBrownRedLow    = checkDefaultSetting(settingType.waveHeight, extraSettings.brown_red.wave_height_low,    gModParams.kRelozu02WaveHeightBrownRedLow,    gWaterColor.brownRed.name .. "." .. gWaterType.waveHeightLow.name)
			gModParams.kRelozu02WaveHeightBrownRedMedium = checkDefaultSetting(settingType.waveHeight, extraSettings.brown_red.wave_height_medium, gModParams.kRelozu02WaveHeightBrownRedMedium, gWaterColor.brownRed.name .. "." .. gWaterType.waveHeightMedium.name)
			gModParams.kRelozu02WaveHeightBrownRedHigh   = checkDefaultSetting(settingType.waveHeight, extraSettings.brown_red.wave_height_high,   gModParams.kRelozu02WaveHeightBrownRedHigh,   gWaterColor.brownRed.name .. "." .. gWaterType.waveHeightHigh.name)
		end
	
		if(extraSettings.foam ~= nil) then
			gModParams.kRelozu02Foam01        = checkMaterial(                               extraSettings.foam.foam_01,         gModParams.kRelozu02Foam01,        gWaterColor.foam.name .. "." .. gWaterColor.foam.name .. "_01")
			gModParams.kRelozu02Foam01HOffset = checkDefaultSetting(settingType.foamHOffset, extraSettings.foam.foam_01_hOffset, gModParams.kRelozu02Foam01HOffset, gWaterColor.foam.name .. "." .. gWaterColor.foam.name .. "_01_hOffset")
			gModParams.kRelozu02Foam02        = checkMaterial(                               extraSettings.foam.foam_02,         gModParams.kRelozu02Foam02,        gWaterColor.foam.name .. "." .. gWaterColor.foam.name .. "_02")
			gModParams.kRelozu02Foam02HOffset = checkDefaultSetting(settingType.foamHOffset, extraSettings.foam.foam_02_hOffset, gModParams.kRelozu02Foam02HOffset, gWaterColor.foam.name .. "." .. gWaterColor.foam.name .. "_02_hOffset")
		end
	end

	if ((type(gModParams) ~= "table") or (type(gExtraSettings) ~= "table")) then
		return
	end

	for a = 1, #gExtraSettings.waterModSettings, 1 do
		local currentSettings = gExtraSettings.waterModSettings[a]
		
		for k,v in pairs(gModParams) do
			if ((type(currentSettings) == "table") and (currentSettings.key == k)) then
				if (v == 1) then
					checkSettings(currentSettings, a)
				end
			end
		end
	end
end

local function createDataTables()
	gDataTableTerrainMaterial = {
		["temperate_dry_fresh"]                                = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02TemperateDryFresh,                                                                 } },
		["temperate_dry_stale"]                                = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02TemperateDryStale,                                                                 } },
		["temperate_dry_small_low"]                            = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02TemperateDrySmall,                                                                 } },
		["temperate_dry_low"]                                  = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02TemperateDryLow,    waveHeight = gModParams.kRelozu02WaveHeightTemperateDryLow,    } },
		["temperate_dry_medium"]                               = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02TemperateDryMedium, waveHeight = gModParams.kRelozu02WaveHeightTemperateDryMedium, } },
		["temperate_dry_high"]                                 = { matColor = gWaterColor.temperateDry.id, matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02TemperateDryHigh,   waveHeight = gModParams.kRelozu02WaveHeightTemperateDryHigh,   } },
		["tropical_fresh"]                                     = { matColor = gWaterColor.tropical.id,     matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02TropicalFresh,                                                                     } },
		["tropical_stale"]                                     = { matColor = gWaterColor.tropical.id,     matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02TropicalStale,                                                                     } },
		["tropical_small_low"]                                 = { matColor = gWaterColor.tropical.id,     matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02TropicalSmall,                                                                     } },
		["tropical_low"]                                       = { matColor = gWaterColor.tropical.id,     matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02TropicalLow,        waveHeight = gModParams.kRelozu02WaveHeightTropicalLow,        } },
		["tropical_medium"]                                    = { matColor = gWaterColor.tropical.id,     matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02TropicalMedium,     waveHeight = gModParams.kRelozu02WaveHeightTropicalMedium,     } },
		["tropical_high"]                                      = { matColor = gWaterColor.tropical.id,     matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02TropicalHigh,       waveHeight = gModParams.kRelozu02WaveHeightTropicalHigh,       } },
		["dark_gray_fresh"]                                    = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02DarkGrayFresh,                                                                     } },
		["res/config/terrain_material/shared/water_dirty.lua"] = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02DarkGrayStale,                                                                     } },
		["dark_gray_small_low"]                                = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02DarkGraySmall,                                                                     } },
		["dark_gray_low"]                                      = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02DarkGrayLow,        waveHeight = gModParams.kRelozu02WaveHeightDarkGrayLow,        } },
		["dark_gray_medium"]                                   = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02DarkGrayMedium,     waveHeight = gModParams.kRelozu02WaveHeightDarkGrayMedium,     } },
		["dark_gray_high"]                                     = { matColor = gWaterColor.darkGray.id,     matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02DarkGrayHigh,       waveHeight = gModParams.kRelozu02WaveHeightDarkGrayHigh,       } },
		["blue_fresh"]                                         = { matColor = gWaterColor.blue.id,         matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02BlueFresh,                                                                         } },
		["blue_stale"]                                         = { matColor = gWaterColor.blue.id,         matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02BlueStale,                                                                         } },
		["blue_small_low"]                                     = { matColor = gWaterColor.blue.id,         matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02BlueSmall,                                                                         } },
		["blue_low"]                                           = { matColor = gWaterColor.blue.id,         matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02BlueLow,            waveHeight = gModParams.kRelozu02WaveHeightBlueLow,            } },
		["blue_medium"]                                        = { matColor = gWaterColor.blue.id,         matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02BlueMedium,         waveHeight = gModParams.kRelozu02WaveHeightBlueMedium,         } },
		["blue_high"]                                          = { matColor = gWaterColor.blue.id,         matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02BlueHigh,           waveHeight = gModParams.kRelozu02WaveHeightBlueHigh,           } },
		["green_blue_fresh"]                                   = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02GreenBlueFresh,                                                                    } },
		["green_blue_stale"]                                   = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02GreenBlueStale,                                                                    } },
		["green_blue_small_low"]                               = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02GreenBlueSmall,                                                                    } },
		["green_blue_low"]                                     = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02GreenBlueLow,       waveHeight = gModParams.kRelozu02WaveHeightGreenBlueLow,       } },
		["green_blue_medium"]                                  = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02GreenBlueMedium,    waveHeight = gModParams.kRelozu02WaveHeightGreenBlueMedium,    } },
		["green_blue_high"]                                    = { matColor = gWaterColor.greenBlue.id,    matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02GreenBlueHigh,      waveHeight = gModParams.kRelozu02WaveHeightGreenBlueHigh,      } },
		["green_fresh"]                                        = { matColor = gWaterColor.green.id,        matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02GreenFresh,                                                                        } },
		["green_stale"]                                        = { matColor = gWaterColor.green.id,        matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02GreenStale,                                                                        } },
		["green_small_low"]                                    = { matColor = gWaterColor.green.id,        matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02GreenSmall,                                                                        } },
		["green_low"]                                          = { matColor = gWaterColor.green.id,        matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02GreenLow,           waveHeight = gModParams.kRelozu02WaveHeightGreenLow,           } },
		["green_medium"]                                       = { matColor = gWaterColor.green.id,        matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02GreenMedium,        waveHeight = gModParams.kRelozu02WaveHeightGreenMedium,        } },
		["green_high"]                                         = { matColor = gWaterColor.green.id,        matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02GreenHigh,          waveHeight = gModParams.kRelozu02WaveHeightGreenHigh,          } },
		["light_brown_fresh"]                                  = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02LightBrownFresh,                                                                   } },
		["light_brown_stale"]                                  = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02LightBrownStale,                                                                   } },
		["light_brown_small_low"]                              = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02LightBrownSmall,                                                                   } },
		["light_brown_low"]                                    = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02LightBrownLow,      waveHeight = gModParams.kRelozu02WaveHeightLightBrownLow,      } },
		["light_brown_medium"]                                 = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02LightBrownMedium,   waveHeight = gModParams.kRelozu02WaveHeightLightBrownMedium,   } },
		["light_brown_high"]                                   = { matColor = gWaterColor.lightBrown.id,   matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02LightBrownHigh,     waveHeight = gModParams.kRelozu02WaveHeightLightBrownHigh,     } },
		["brown_red_fresh"]                                    = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.fresh.id,  params = { available = gModParams.kRelozu02BrownRedFresh,                                                                     } },
		["brown_red_stale"]                                    = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.stale.id,  params = { available = gModParams.kRelozu02BrownRedStale,                                                                     } },
		["brown_red_small_low"]                                = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.small.id,  params = { available = gModParams.kRelozu02BrownRedSmall,                                                                     } },
		["brown_red_low"]                                      = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.low.id,    params = { available = gModParams.kRelozu02BrownRedLow,        waveHeight = gModParams.kRelozu02WaveHeightBrownRedLow,        } },
		["brown_red_medium"]                                   = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.medium.id, params = { available = gModParams.kRelozu02BrownRedMedium,     waveHeight = gModParams.kRelozu02WaveHeightBrownRedMedium,     } },
		["brown_red_high"]                                     = { matColor = gWaterColor.brownRed.id,     matType = gWaterType.high.id,   params = { available = gModParams.kRelozu02BrownRedHigh,       waveHeight = gModParams.kRelozu02WaveHeightBrownRedHigh,       } },
		["foam_01"]                                            = { matColor = gWaterColor.foam.id,         matType = "01",                 params = { available = gModParams.kRelozu02Foam01,             hOffset    = gModParams.kRelozu02Foam01HOffset,                } },
		["foam_02"]                                            = { matColor = gWaterColor.foam.id,         matType = "02",                 params = { available = gModParams.kRelozu02Foam02,             hOffset    = gModParams.kRelozu02Foam02HOffset,                } },
	}
end

local function addFiltersAndModifiers()
	local darkGrayStaleName = _("hTerrainMatDarkGrayStale")

	-- Terrain materials
	addModifier("loadTerrainMaterial", function (fileName, data)
		if ((data.relozuWaterMatId == nil) and (fileName ~= "res/config/terrain_material/shared/water_dirty.lua")) then
			return data
		elseif (fileName ~= "res/config/terrain_material/shared/water_dirty.lua") then
			fileName = data.relozuWaterMatId
		end

		local item = gDataTableTerrainMaterial[fileName]
		
		-- Prevents also modifying of water dirty
		if ((item == nil) or (item.params.available == 0)) then
			return data
		end

		if ((item.params.waveHeight ~= nil) and (item.params.waveHeight == 0)) then
			data.detailNormalTexture = "terrain/relozu_terrain_material_water/water_small_nrml.dds"
		end
		
		if (item.matColor == gWaterColor.foam.id) then
			data.hOffset = item.params.hOffset * .1
		elseif ((item.matColor == gWaterColor.darkGray.id) and (item.matType == gWaterType.stale.id)) then
			data.name            = darkGrayStaleName
			data.detailSize      = 1 / 32
			data.overlaySize     = .05
			data.overlayStrength = .05
			data.priority        = 1000000
		end

		data.order = gOrderBaseValue + (item.matColor * 20) + item.matType

		return data
	end)

	if (gHiddenSettings.allowFilteringWaterDarkGrayStale == false) then
		addFileFilter("terrainMaterial", function (fileName, data)
			if (data.relozuWaterMatId == nil) then
				return true
			end
	
			local item = gDataTableTerrainMaterial[data.relozuWaterMatId]
			
			if (item == nil) then
				return true
			end
	
			if (item.params.available == 0) then
				return false
			end
	
			return true
		end)
	else
		addFileFilter("terrainMaterial", function (fileName, data)
			if (data.relozuWaterMatId ~= nil) then
				fileName = data.relozuWaterMatId
			elseif (fileName ~= "res/config/terrain_material/shared/water_dirty.lua") then
				return true
			end
	
			local item = gDataTableTerrainMaterial[fileName]
			
			if (item == nil) then
				return true
			end
	
			if (item.params.available == 0) then
				return false
			end
	
			return true
		end)
	end
end

-----------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------- GLOBAL ATTRIBUTES AND FUNCTIONS ----------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------------------------------------------------

-- runFn
function relozu_terrain_material_water.applySettings(modParams)
	prepareModParams(modParams)
	adoptExtraSettings()
	createDataTables()
	addFiltersAndModifiers()
end

function relozu_terrain_material_water.makeModParams(extraModSettings)
	local modParams = {
		{ key = "kRelozu02TemperateDryFresh",            name = _("hTemperateDryFresh"),          tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TemperateDryStale",            name = _("hTemperateDryStale"),          tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TemperateDrySmall",            name = _("hTemperateDrySmall"),          tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02TemperateDryLow",              name = _("hTemperateDryLow"),            tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TemperateDryMedium",           name = _("hTemperateDryMedium"),         tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TemperateDryHigh",             name = _("hTemperateDryHigh"),           tooltip = _("nTemperateDry"),          uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTemperateDryLow",    name = _("hTemperateDryDefaultLow"),     tooltip = _("nTemperateDry"),          uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTemperateDryMedium", name = _("hTemperateDryDefaultMedium"),  tooltip = _("nTemperateDry"),          uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTemperateDryHigh",   name = _("hTemperateDryDefaultHigh"),    tooltip = _("nTemperateDry"),          uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02TropicalFresh",                name = _("hTropicalFresh"),              tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TropicalStale",                name = _("hTropicalStale"),              tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TropicalSmall",                name = _("hTropicalSmall"),              tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02TropicalLow",                  name = _("hTropicalLow"),                tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TropicalMedium",               name = _("hTropicalMedium"),             tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02TropicalHigh",                 name = _("hTropicalHigh"),               tooltip = _("nTropical"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTropicalLow",        name = _("hTropicalDefaultLow"),         tooltip = _("nTropical"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTropicalMedium",     name = _("hTropicalDefaultMedium"),      tooltip = _("nTropical"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightTropicalHigh",       name = _("hTropicalDefaultHigh"),        tooltip = _("nTropical"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02DarkGrayFresh",                name = _("hDarkGrayFresh"),              tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02DarkGrayStale",                name = _("hDarkGrayStale"),              tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02DarkGraySmall",                name = _("hDarkGraySmall"),              tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02DarkGrayLow",                  name = _("hDarkGrayLow"),                tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02DarkGrayMedium",               name = _("hDarkGrayMedium"),             tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02DarkGrayHigh",                 name = _("hDarkGrayHigh"),               tooltip = _("nDarkGray"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02WaveHeightDarkGrayLow",        name = _("hDarkGrayDefaultLow"),         tooltip = _("nDarkGray"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightDarkGrayMedium",     name = _("hDarkGrayDefaultMedium"),      tooltip = _("nDarkGray"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightDarkGrayHigh",       name = _("hDarkGrayDefaultHigh"),        tooltip = _("nDarkGray"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02BlueFresh",                    name = _("hBlueFresh"),                  tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BlueStale",                    name = _("hBlueStale"),                  tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BlueSmall",                    name = _("hBlueSmall"),                  tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02BlueLow",                      name = _("hBlueLow"),                    tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BlueMedium",                   name = _("hBlueMedium"),                 tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BlueHigh",                     name = _("hBlueHigh"),                   tooltip = _("nBlue"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBlueLow",            name = _("hBlueDefaultLow"),             tooltip = _("nBlue"),                  uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBlueMedium",         name = _("hBlueDefaultMedium"),          tooltip = _("nBlue"),                  uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBlueHigh",           name = _("hBlueDefaultHigh"),            tooltip = _("nBlue"),                  uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02GreenBlueFresh",               name = _("hGreenBlueFresh"),             tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenBlueStale",               name = _("hGreenBlueStale"),             tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenBlueSmall",               name = _("hGreenBlueSmall"),             tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02GreenBlueLow",                 name = _("hGreenBlueLow"),               tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenBlueMedium",              name = _("hGreenBlueMedium"),            tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenBlueHigh",                name = _("hGreenBlueHigh"),              tooltip = _("nGreenBlue"),             uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenBlueLow",       name = _("hGreenBlueDefaultLow"),        tooltip = _("nGreenBlue"),             uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenBlueMedium",    name = _("hGreenBlueDefaultMedium"),     tooltip = _("nGreenBlue"),             uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenBlueHigh",      name = _("hGreenBlueDefaultHigh"),       tooltip = _("nGreenBlue"),             uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02GreenFresh",                   name = _("hGreenFresh"),                 tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenStale",                   name = _("hGreenStale"),                 tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenSmall",                   name = _("hGreenSmall"),                 tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02GreenLow",                     name = _("hGreenLow"),                   tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenMedium",                  name = _("hGreenMedium"),                tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02GreenHigh",                    name = _("hGreenHigh"),                  tooltip = _("nGreen"),                 uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenLow",           name = _("hGreenDefaultLow"),            tooltip = _("nGreen"),                 uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenMedium",        name = _("hGreenDefaultMedium"),         tooltip = _("nGreen"),                 uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightGreenHigh",          name = _("hGreenDefaultHigh"),           tooltip = _("nGreen"),                 uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02LightBrownFresh",              name = _("hLightBrownFresh"),            tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02LightBrownStale",              name = _("hLightBrownStale"),            tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02LightBrownSmall",              name = _("hLightBrownSmall"),            tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02LightBrownLow",                name = _("hLightBrownLow"),              tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02LightBrownMedium",             name = _("hLightBrownMedium"),           tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02LightBrownHigh",               name = _("hLightBrownHigh"),             tooltip = _("nLightBrown"),            uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightLightBrownLow",      name = _("hLightBrownDefaultLow"),       tooltip = _("nLightBrown"),            uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightLightBrownMedium",   name = _("hLightBrownDefaultMedium"),    tooltip = _("nLightBrown"),            uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightLightBrownHigh",     name = _("hLightBrownDefaultHigh"),      tooltip = _("nLightBrown"),            uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02BrownRedFresh",                name = _("hBrownRedFresh"),              tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BrownRedStale",                name = _("hBrownRedStale"),              tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BrownRedSmall",                name = _("hBrownRedSmall"),              tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = gDefaultParam, },
		{ key = "kRelozu02BrownRedLow",                  name = _("hBrownRedLow"),                tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BrownRedMedium",               name = _("hBrownRedMedium"),             tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02BrownRedHigh",                 name = _("hBrownRedHigh"),               tooltip = _("nBrownRed"),              uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBrownRedLow",        name = _("hBrownRedDefaultLow"),         tooltip = _("nBrownRed"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBrownRedMedium",     name = _("hBrownRedDefaultMedium"),      tooltip = _("nBrownRed"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },
		{ key = "kRelozu02WaveHeightBrownRedHigh",       name = _("hBrownRedDefaultHigh"),        tooltip = _("nBrownRed"),              uiType = "BUTTON",   values = gParamsWaveHeight,           defaultIndex = 1, },

		{ key = "kRelozu02Foam01",                       name = _("hFoam01"),                     tooltip = _("nFoam"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02Foam01HOffset",                name = _("hFoam01HOffset"),              tooltip = _("tFoamHOffset"),           uiType = "SLIDER",   values = gParamsFoamHOffset,          defaultIndex = 0, },
		{ key = "kRelozu02Foam02",                       name = _("hFoam02"),                     tooltip = _("nFoam"),                  uiType = "CHECKBOX", values = { "false", "true" },         defaultIndex = 1, },
		{ key = "kRelozu02Foam02HOffset",                name = _("hFoam02HOffset"),              tooltip = _("tFoamHOffset"),           uiType = "SLIDER",   values = gParamsFoamHOffset,          defaultIndex = 0, },
	}

	local function checkParam(key, param, index)
		local finalParam = {
			key = "kRelozu02Extra_Undefined",
			name = _("hUndefinedParamName"),
			tooltip = _("hUndefinedParamTooltip"),
			uiType = "CHECKBOX",
			values = { "false", "true" },
			defaultIndex = 0
		}
	
		--[[
			Expected structure:
			{
				name    = "Example 1",
				tooltip = "Tooltip of example 1",
				default = false
			},
		]]

		if (type(key) == "string") then
			finalParam.key = key
		elseif (key ~= nil) then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. index .. "].key (Index: " .. index .. ") - Expected type: string - Get: " .. type(key) .. gErrMsgIgnoreTable)
			return nil
		else
			printWarningMessage(gExtraSettingsFile .. "Wrong or no entry for waterModSettings[" .. index .. "].key (Index: " .. index .. ") - A key is necessary." .. gErrMsgIgnoreTable)
			return nil
		end

		if (type(param) == "nil") then
			printWarningMessage(gExtraSettingsFile .. "Wrong or no entry for waterModSettings[" .. index .. "].param (Index: " .. index .. ") - A param table is necessary. - A default param table will be used as fallback.")
			param = { }
		elseif (type(param) ~= "table") then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. index .. "].param (Index: " .. index .. ") - A param table is necessary. - A default param table will be used as fallback.")
			param = { }
		end

		if (type(param.name) == "string") then
			finalParam.name = param.name
		elseif (param.name ~= nil) then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. index .. "].param.name (Index: " .. index ..", Key: \"" .. finalParam.key .. "\") - Expected type: string - Get: " .. type(param.name) .. " - Replaced with: \"" .. finalParam.name .. "\"")
		end

		if (type(param.tooltip) == "string") then
			finalParam.tooltip = param.tooltip
		elseif (param.tooltip ~= nil) then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. index .. "].param.tooltip (Index: " .. index ..", Key: \"" .. finalParam.key .. "\") - Expected type: string - Get: " .. type(param.tooltip) .. " - Replaced with: \"" .. finalParam.tooltip .. "\"")
		end

		if (type(param.default) == "boolean") then
			if (param.default) then
				finalParam.defaultIndex = 1
			else
				finalParam.defaultIndex = 0
			end
		elseif (param.default ~= nil) then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for waterModSettings[" .. index .. "].param.default (Index: " .. index ..", Key: \"" .. finalParam.key .. "\") - Expected type: boolean - Get: " .. type(param.default) .. " - Replaced with: false")
		end
		
		return finalParam
	end

	setModId(getCurrentModId())

	if (type(extraModSettings) == "table") then
		gExtraSettings = table.copy(extraModSettings)
	else
		gExtraSettings = nil
	end

	-- check of extraSettings tables
	if ((type(gExtraSettings) ~= "table") or (type(gExtraSettings.waterModSettings) ~= "table")) then
		gExtraSettings = nil
		return modParams
	else
		local extraModParams = { }
		local extraSettingsToRemove = { }

		-- add params of extra settings to modParams, if a key does not already exist
		for a = 1, #gExtraSettings.waterModSettings, 1 do
			local currentSettings = gExtraSettings.waterModSettings[a]

			if (type(currentSettings) == "table") then
				local bIgnore = false
				local param   = checkParam(currentSettings.key, currentSettings.param, a)

				if (param == nil) then
					bIgnore = true
				end
				
				if (bIgnore == false) then
					for b = 1, #extraModParams, 1 do
						if (param.key == extraModParams[b].key) then
							bIgnore = true
							printWarningMessage(gExtraSettingsFile .. "waterModSettings[" .. a .. "].key (Index: " .. a .. ") - The key \"" .. param.key .. "\" is used already. Overwriting is not allowed." .. gErrMsgIgnoreTable)
						end
					end
				end

				if (bIgnore == false) then
					for b = 1, #modParams, 1 do
						if (param.key == modParams[b].key) then
							bIgnore = true
							printWarningMessage(gExtraSettingsFile .. "waterModSettings[" .. a .. "].key (Index: " .. a .. ") - modParams already contains the key \"" .. param.key .. "\". Overwriting is not allowed." .. gErrMsgIgnoreTable)
						end
					end
				end

				if (bIgnore == false) then
					currentSettings.key                 = param.key
					currentSettings.param               = param
					extraModParams[#extraModParams + 1] = param
				else
					extraSettingsToRemove[#extraSettingsToRemove + 1] = a
				end
			else
				printWarningMessage(gExtraSettingsFile .. "waterModSettings[" .. a .. "] (Index: " .. a .. ") - Wrong settings entry, a settings entry must be a table. Get: " .. type(currentSettings))
			end
		end

		for a = #extraSettingsToRemove, 1, -1 do
			table.remove(gExtraSettings.waterModSettings, extraSettingsToRemove[a])
		end

		-- hide default mod params?
		if ((type(gExtraSettings.hide_default_mod_settings) == "boolean") and gExtraSettings.hide_default_mod_settings and (#extraModParams > 0)) then
			modParams = { }
		elseif ((type(gExtraSettings.hide_default_mod_settings) ~= "nil") and (type(gExtraSettings.hide_default_mod_settings) ~= "boolean")) then
			printWarningMessage(gExtraSettingsFile .. "Wrong entry for hide_default_mod_settings - Expected type: boolean - Get: " .. type(gExtraSettings.hide_default_mod_settings) .. gErrMsgIgnoreSetting)
		end

		for a = 1, #extraModParams, 1 do
			modParams[#modParams + 1] = extraModParams[a]
		end
	end

	if (#modParams == 0) then
		modParams = nil
	end

	return modParams
end

-- postRunFn
function relozu_terrain_material_water.clearData()
	gDataTableTerrainMaterial = nil	
	gErrMsgIgnoreSetting      = nil
	
	gModId                    = nil
	gModParams                = nil
	gExtraSettings            = nil
	gOrderBaseValue           = nil
	gDefaultParam             = nil
	gWaterColor               = nil
	gWaterType                = nil
	gExtraSettingsFile        = nil
	gHiddenSettings           = nil

	gParamsFoamHOffset        = nil
	gParamsWaveHeight         = nil
end

return relozu_terrain_material_water
